//-----------------------------------------------//
//                                               //
//  Arch Macros                                  //
//                                               //
//  Author: Bonsai (mail: bonsai@b0n541.net)     //
//                                               //
//  Copyright: GNU General Public License, 2001  //
//                                               //
//-----------------------------------------------//

//-----------------------------------------------------------------------------------------------------------------------------------------------------------//
//
//  Macro for normal brick arches
//
//  Usage: Arch (span, inner arch height, outer arch height, brick height, minimal inner splice, front pattern, mortar depth, brick texture, mortar texture)
//        
//      Span                    : distance between the points the arch is laying over
//      inner arch height       : height over points where the arch is laying over
//      outer arch height       : height over points where the arch is laying over plus height of the arch itself 
//      brick height            : is needed to calculate the number of layers needed
//      minimal inner splice    : minimal splice at the inner side of the arch
//      front pattern           : 0 = userdefined object
//                                1 = only one brick is seen on the front
//                                2 = there is alternating one or two half bricks seen on the front
//                                3 = there is alternating two three-quater and three half bricks seen on the front
//                                4 = wedge brick front pattern
//      userdefined object      : object definition for user defined object
//      mortar depth            : how much the brick stands out of the mortar
//      brick texture           : texture definition for the bricks
//      mortar texture          : texture definition for the mortar
//
//-----------------------------------------------------------------------------------------------------------------------------------------------------------//

#macro Arch (Span, InnerArchHeight, OuterArchHeight, BrickHeight, MinInnerSplice, FrontPattern, MortarDepth, LocalBrickTexture, LocalMortarTexture)
        // parameter checks
        #if (Span < 0)
                #warning "Span can't be negative."
        #else #if (InnerArchHeight <= 0)
                #warning "Inner arch height can't be negative or null."
        #else #if (OuterArchHeight <= 0)
                #warning "Outer arch height can't be negative or null."
        #else #if (InnerArchHeight > OuterArchHeight)
                #warning "Outer arch height must be greater than inner arch height."
        #else #if (BrickHeight <= 0)
                #warning "Brick height can't be negative or null."
        #else #if (MinInnerSplice <= 0)
                #warning "Minimal inner splice can't be negative or null."
        #else #if ((FrontPattern < 1) | (FrontPattern > 4))
                #warning "Front pattern must be 1, 2, 3 or 4."
        #else #if (MortarDepth < 0))
                #warning "Mortar depth can't be negative."
        #end #end #end #end #end #end #end #end
        
        #declare InnerArchRadius = (InnerArchHeight / 2) + pow (Span, 2) / (8 * InnerArchHeight); 
        #declare AngleAtCenterPoint = 2 * degrees (acos ((InnerArchRadius - InnerArchHeight) / InnerArchRadius));
        #declare InnerLength = 2 * InnerArchRadius * pi * AngleAtCenterPoint / 360;
        #declare OuterLength = (2 * InnerArchRadius + 2 * BrickHeight) * pi * AngleAtCenterPoint / 360;
        #declare NumberOfLayers = div (InnerLength - MinInnerSplice, BrickHeight + MinInnerSplice);
        //number of layers must alway be uneven
        #if (mod (NumberOfLayers, 2) = 0)
                #declare NumberOfLayers = NumberOfLayers - 1;
        #end

        // the brick for one or two half bricks on the front
        #declare TheBrick = box
                {
                <0, 0, 0>,
                <OuterArchHeight - InnerArchHeight, BrickHeight, (OuterArchHeight - InnerArchHeight - 1) / 2>
                }
                
        // pattern for one brick front
        #declare OneBrickFront = union
                {
                object
                        {
                        TheBrick
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, 0, 0>
                        }
                object
                        {
                        TheBrick
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, 0, (OuterArchHeight - InnerArchHeight - 1) / 2 + 1>
                        }                        
                }
                
        // pattern for two half bricks front
        #declare TwoHalfBricksFront = union
                {
                object
                        {
                        TheBrick
                        rotate <0, 0, 90>
                        rotate <90, 0, 0>
                        translate <BrickHeight / 2, (OuterArchHeight - InnerArchHeight - 1) / 2, 0>
                        }
                object
                        {
                        TheBrick
                        rotate <0, 0, 90>
                        rotate <90, 0, 0>
                        translate <BrickHeight / 2, OuterArchHeight - InnerArchHeight, 0>
                        }                        
                }        
        
        // three-quater brick for the three-quater front pattern
        #declare ThreeQuaterBrick = box
                {
                <0, 0, 0>,
                <(OuterArchHeight - InnerArchHeight - 1) / 2, BrickHeight, (OuterArchHeight - InnerArchHeight - 2) / 3>
                }
                
        // pattern for two three-quater brick front
        #declare TwoThreeQuaterBrickFront = union
                {
                object
                        {
                        ThreeQuaterBrick
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, 0, 0>
                        }
                object
                        {
                        ThreeQuaterBrick
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, (OuterArchHeight - InnerArchHeight - 1) / 2 + 1, 0>
                        }
                object
                        {
                        ThreeQuaterBrick
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, 0, (OuterArchHeight - InnerArchHeight - 2) / 3 + 1>
                        }
                object
                        {
                        ThreeQuaterBrick
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, (OuterArchHeight - InnerArchHeight - 1) / 2 + 1, (OuterArchHeight - InnerArchHeight - 2) / 3 + 1>
                        }
                }
        
        // half-brick for the three-quater front pattern
        #declare BrickForThreeQuaterFront = box
                {
                <0, 0, 0>,
                <2 * (OuterArchHeight - InnerArchHeight - 2) / 3 + 1, BrickHeight, (OuterArchHeight - InnerArchHeight - 2) / 3>
                }
                
        #declare ThreeHalfBrickFront = union
                {
                object
                        {
                        BrickForThreeQuaterFront
                        rotate <0, 0, 90>
                        rotate <90, 0, 0>
                        translate <BrickHeight / 2, (OuterArchHeight - InnerArchHeight - 2) / 3, 0>
                        }
                object
                        {
                        BrickForThreeQuaterFront
                        rotate <0, 0, 90>
                        rotate <90, 0, 0>
                        translate <BrickHeight / 2, 2 * (OuterArchHeight - InnerArchHeight - 2) / 3 + 1, 0>
                        }
                object
                        {
                        BrickForThreeQuaterFront
                        rotate <0, 0, 90>
                        rotate <90, 0, 0>
                        translate <BrickHeight / 2, 3 * (OuterArchHeight - InnerArchHeight - 2) / 3 + 2, 0>
                        }
                }                
        
        // brick for one and a half brick front pattern
        #declare TheBrick2 = box
                {
                <0, 0, 0>,
                <2 * (OuterArchHeight - InnerArchHeight - 2) / 3 + 1, BrickHeight, (OuterArchHeight - InnerArchHeight - 2) / 3>
                }
        
        // pattern 1 for one and a half brick front        
        #declare OneAndHalfBrickFront1 = union
                {
                object
                        {
                        TheBrick2
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, 0, 0>
                        }
                object
                        {
                        TheBrick2
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, 0, (OuterArchHeight - InnerArchHeight - 2) / 3 + 1>
                        }                        
                object
                        {
                        TheBrick2
                        rotate <0, 0, 90>
                        rotate <90, 0, 0>
                        translate <BrickHeight / 2, 3 * (OuterArchHeight - InnerArchHeight - 2) / 3 + 2, 0>
                        }                        
                }

        // pattern 2 for one and a half brick front        
        #declare OneAndHalfBrickFront2 = union
                {
                object
                        {
                        TheBrick2
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, ((OuterArchHeight - InnerArchHeight - 2) / 3) + 1, 0>
                        }
                object
                        {
                        TheBrick2
                        rotate <0, 0, 90>
                        translate <BrickHeight / 2, ((OuterArchHeight - InnerArchHeight - 2) / 3) + 1, (OuterArchHeight - InnerArchHeight - 2) / 3 + 1>
                        }                        
                object
                        {
                        TheBrick2
                        rotate <0, 0, 90>
                        rotate <90, 0, 0>
                        translate <BrickHeight / 2, (OuterArchHeight - InnerArchHeight - 2) / 3, 0>
                        }                        
                }

        #declare InnerSplice = (InnerLength - NumberOfLayers * BrickHeight) / (NumberOfLayers + 1);

        // Rotation angle
        #if (Span > 0) 
                // calculation for arch
                #declare RotationAngle = ((AngleAtCenterPoint - (InnerSplice / InnerArchRadius)) / NumberOfLayers) - 2 * (InnerSplice / InnerArchRadius);
        #else
                // calculation for circle
                #declare RotationAngle = AngleAtCenterPoint / NumberOfLayers;
        #end
        
        // Layers per side
        #declare LayersPerSide = (NumberOfLayers - 1) / 2;
        
        // Building the arch
        union
                {
                //king stone
                #if (FrontPattern = 3)
                        object
                                {
                                TwoThreeQuaterBrickFront
                                translate <0, InnerArchRadius, 0>
                                texture {LocalBrickTexture}
                                }
                #else 
                        #if (FrontPattern = 4)
                                object
                                        {
                                        OneAndHalfBrickFront1
                                        translate <0, InnerArchRadius, 0>
                                        texture {LocalBrickTexture}
                                        }
                        #else
                                object
                                        {
                                        OneBrickFront
                                        translate <0, InnerArchRadius, 0>
                                        texture {LocalBrickTexture}
                                        }
                        #end
                #end
                                
                //all other bricks
                #declare CurrentLayer = 1;
                #while (CurrentLayer <= LayersPerSide)
                        #declare CurrentAngle = RotationAngle * CurrentLayer;
                        #switch (FrontPattern)
                        #case (0)
                        #break
                        #case (1)
                                // front pattern 1
                                object
                                        {
                                        OneBrickFront
                                        translate <0, InnerArchRadius, 0>
                                        rotate <0, 0, CurrentAngle>
                                        texture {LocalBrickTexture}
                                        }
                                object
                                        {
                                        OneBrickFront
                                        translate <0, InnerArchRadius, 0>
                                        rotate <0, 0, -CurrentAngle>
                                        texture {LocalBrickTexture}
                                        }
                        #break
                        #case (2)
                                // front pattern 2
                                #if (mod(CurrentLayer, 2) = 0)
                                        object
                                                {
                                                OneBrickFront
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                        object
                                                {
                                                OneBrickFront
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, -CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                #else
                                        object
                                                {
                                                TwoHalfBricksFront
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                        object
                                                {
                                                TwoHalfBricksFront
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, -CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                #end
                        #break
                        #case (3)
                                // front pattern 3
                                #if (mod(CurrentLayer, 2) = 0)
                                        object
                                                {
                                                TwoThreeQuaterBrickFront
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                        object
                                                {
                                                TwoThreeQuaterBrickFront
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, -CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                #else
                                        object
                                                {
                                                ThreeHalfBrickFront
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                        object
                                                {
                                                ThreeHalfBrickFront
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, -CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                #end
                        #break
                        #case (4)
                                // front pattern 4
                                #if (mod(CurrentLayer, 2) = 0)
                                        object
                                                {
                                                OneAndHalfBrickFront1
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                        object
                                                {
                                                OneAndHalfBrickFront1
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, -CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                #else
                                        object
                                                {
                                                OneAndHalfBrickFront2
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                        object
                                                {
                                                OneAndHalfBrickFront2
                                                translate <0, InnerArchRadius, 0>
                                                rotate <0, 0, -CurrentAngle>
                                                texture {LocalBrickTexture}
                                                }
                                #end
                        #end
                        #declare CurrentLayer = CurrentLayer + 1;
                #end
                
                //Mortar
                #if (MortarDepth = 0)
                        #declare MortarDepth = 0.0000001;
                #end
                
                #declare OuterArchRadius = InnerArchRadius + OuterArchHeight - InnerArchHeight;

                object
                        {
                        difference
                                {
                                difference
                                        {
                                        #if ((FrontPattern = 3) | (FrontPattern = 4))
                                                #declare ArchDepth = 2 * (OuterArchHeight - InnerArchHeight - 2) / 3 + 1;
                                        #else
                                                #declare ArchDepth = OuterArchHeight - InnerArchHeight;
                                        #end
                                        cylinder
                                                {
                                                <0, 0, MortarDepth>,
                                                <0, 0, ArchDepth - MortarDepth>,
                                                sqrt (pow (OuterArchRadius, 2) + pow ((BrickHeight / 2), 2)) + 1
                                                }
                                        
                                        cylinder
                                                {
                                                <0, 0, MortarDepth - 1>,
                                                <0, 0, ArchDepth - MortarDepth + 1>,
                                                sqrt (pow (InnerArchRadius, 2) + pow ((BrickHeight / 2), 2)) + MortarDepth 
                                                }
                                        }                                        
                                
                                #if (AngleAtCenterPoint < 180)
                                        merge
                                                {
                                                box
                                                        {
                                                        <-OuterArchRadius - 2, 0, -1>,
                                                        <OuterArchRadius + 2, -OuterArchRadius - 2, ArchDepth + 1>
                                                        rotate <0, 0, (180 - AngleAtCenterPoint) / 2>
                                                        }
                                                box
                                                        {
                                                        <-OuterArchRadius - 2, 0, -1>,
                                                        <OuterArchRadius + 2, -OuterArchRadius - 2, ArchDepth + 1>
                                                        rotate <0, 0, -(180 - AngleAtCenterPoint) / 2>
                                                        }
                                                }
                                #else
                                        #if (AngleAtCenterPoint = 180)
                                                box
                                                        {
                                                        <-OuterArchRadius - 2, 0, -1>,
                                                        <OuterArchRadius + 2, -OuterArchRadius - 2, ArchDepth + 1>
                                                        }
                                        #else
                                                #if (AngleAtCenterPoint < 360)
                                                        intersection
                                                                {
                                                                box
                                                                        {
                                                                        <-OuterArchRadius - 2, 0, -1>,
                                                                        <OuterArchRadius + 2, -OuterArchRadius - 2, ArchDepth + 1>
                                                                        rotate <0, 0, (180 - AngleAtCenterPoint) / 2>
                                                                        }
                                                                box
                                                                        {
                                                                        <-OuterArchRadius - 2, 0, -1>,
                                                                        <OuterArchRadius + 2, -OuterArchRadius - 2, ArchDepth + 1>
                                                                        rotate <0, 0, -(180 - AngleAtCenterPoint) / 2>
                                                                        }
                                                                }
                                                #end                                                        
                                        #end
                                #end
                                }
                                
                                
                        texture {LocalMortarTexture}
                        }
                                
                // move arch to the origin <0, 0, 0>
                translate <Span / 2, -InnerArchRadius + InnerArchHeight, 0>
                }
#end
//------------------------------------------//
//                                          //
//  End of Macro for standard brick arches  //
//                                          //
//------------------------------------------//


//--------------------------------------------------------------------------------------------------------------------------------------------//
//
//  Macro for wedge brick arches
//
//  Usage: arch WedgeBrickArch (span, inner arch height, outer arch height, number of layers, minimal inner splice, arch depth, brick texture)
//        
//      span                    : distance between the points the arch is laying over
//      inner arch height       : height over points where the arch is laying over
//      outer arch height       : height over points where the arch is laying over plus height of the arch itself 
//      number of layers        : is needed to calculate the the size of the bricks
//      minimal inner splice    : minimal splice at the inner side of the arch
//      arch depth              : depth of the arch (z-direction) 
//      brick texture           : texture definition for the bricks
//
//--------------------------------------------------------------------------------------------------------------------------------------------//

#macro WedgeBrickArch (Span, InnerArchHeight, OuterArchHeight, NumberOfLayers, MinInnerSplice, ArchDepth, LocalBrickTexture)
        // parameter checks
        #if (Span < 0)
                #warning "Span can't be negative."
        #else #if (InnerArchHeight <= 0)
                #warning "Inner arch height can't be negative or null."
        #else #if (OuterArchHeight <= 0)
                #warning "Outer arch height can't be negative or null."
        #else #if (InnerArchHeight > OuterArchHeight)
                #warning "Outer arch height must be greater than inner arch height."
        #else #if (NumberOfLayers < 3)
                #warning "Number of layers must be at least 3."
        #else #if (MinInnerSplice <= 0)
                #warning "Minimal inner splice can't be negative or null."
        #else #if (ArchDepth <= 0)
                #warning "Arch depth can't be negative or null."
        #end #end #end #end #end #end #end
        
        #if (mod (NumberOfLayers, 2) = 0)
                #warning concat("Number of layers should be uneven. Using ", str(NumberOfLayers - 1, 5, 0), " layers instead of ", str(NumberOfLayers, 5, 0), ".")
                #declare NumberOfLayers = NumberOfLayers - 1;
        #end
        
        #declare InnerArchRadius = (InnerArchHeight / 2) + pow (Span, 2) / (8 * InnerArchHeight); 
        #declare AngleAtCenterPoint = 2 * degrees (acos ((InnerArchRadius - InnerArchHeight) / InnerArchRadius));
        #declare InnerLength = 2 * InnerArchRadius * pi * AngleAtCenterPoint / 360;

        #if (Span > 0) 
                // calculation for arch
                #declare WedgeBrickInnerLengthRound = (InnerLength - NumberOfLayers * MinInnerSplice - MinInnerSplice) / NumberOfLayers;
        #else
                // calculation for circle
                #declare WedgeBrickInnerLengthRound = (InnerLength - NumberOfLayers * MinInnerSplice) / NumberOfLayers;
        #end

        #declare BrickHeight = 2 * InnerArchRadius * sin (0.5 * (WedgeBrickInnerLengthRound / InnerArchRadius));
        #declare OuterArchRadius = InnerArchRadius + OuterArchHeight - InnerArchHeight;
        #declare OuterLength = 2 * OuterArchRadius * pi * AngleAtCenterPoint / 360;

        #if (Span > 0) 
                // calculation for arch
                #declare WedgeBrickOuterLengthRound = (OuterLength - NumberOfLayers * MinInnerSplice - MinInnerSplice) / NumberOfLayers;
        #else
                // calculation for circle
                #declare WedgeBrickOuterLengthRound = (OuterLength - NumberOfLayers * MinInnerSplice) / NumberOfLayers;
        #end
        #declare WedgeBrickOuterLengthStraight = 2 * OuterArchRadius * sin (0.5 * (WedgeBrickOuterLengthRound / OuterArchRadius));
        #declare WedgeBrickOuterHeightStraight = 0.5 * sqrt (4 * pow (OuterArchRadius, 2) - pow (WedgeBrickOuterLengthStraight, 2));
        #declare WedgeBrickInnerLengthStraight = BrickHeight;
        #declare WedgeBrickInnerHeightStraight = 0.5 * sqrt (4 * pow (InnerArchRadius, 2) - pow (WedgeBrickInnerLengthStraight, 2));

        // brick for the wedge brick front pattern
        #declare WedgeBrick = prism
                {
                linear_spline
                -ArchDepth, 0, 5,
                <-WedgeBrickInnerLengthStraight / 2, 0>,
                <WedgeBrickInnerLengthStraight / 2, 0>,
                <WedgeBrickOuterLengthStraight / 2, WedgeBrickOuterHeightStraight - WedgeBrickInnerHeightStraight>,
                <-WedgeBrickOuterLengthStraight / 2, WedgeBrickOuterHeightStraight - WedgeBrickInnerHeightStraight>
                <-WedgeBrickInnerLengthStraight / 2, 0>
                rotate <-90, 0, 0>
                }
                
        #declare WedgeBrickFront = union
                {
                object
                        {
                        WedgeBrick
                        }
                }
        
        #declare InnerSplice = (InnerLength - NumberOfLayers * WedgeBrickInnerLengthRound) / (NumberOfLayers + 1);

        #if (Span > 0) 
                // calculation for arch
                #declare RotationAngle = ((AngleAtCenterPoint - (InnerSplice / InnerArchRadius)) / NumberOfLayers) - 2 * (InnerSplice / InnerArchRadius);
        #else
                // calculation for circle
                #declare RotationAngle = AngleAtCenterPoint / NumberOfLayers;
        #end
        
        #declare LayersPerSide = (NumberOfLayers - 1) / 2;
        
        union
                {
                //king stone
                object
                        {
                        WedgeBrickFront
                        translate <0, WedgeBrickInnerHeightStraight, 0>
                        texture {LocalBrickTexture}
                        }
                                
                //all other bricks
                #declare CurrentLayer = 1;
                #while (CurrentLayer <= LayersPerSide)
                        #declare CurrentAngle = RotationAngle * CurrentLayer;
                        // wedge brick front
                        object
                                {
                                WedgeBrickFront
                                translate <0, WedgeBrickInnerHeightStraight, 0>
                                rotate <0, 0, CurrentAngle>
                                texture {LocalBrickTexture}
                                }
                        object
                                {
                                WedgeBrickFront
                                translate <0, WedgeBrickInnerHeightStraight, 0>
                                rotate <0, 0, -CurrentAngle>
                                texture {LocalBrickTexture}
                                }
                        #declare CurrentLayer = CurrentLayer + 1;
                #end

                // move arch to the origin <0, 0, 0>
                translate <Span / 2, -InnerArchRadius + InnerArchHeight, 0>
                }
#end
//---------------------------------------//
//                                       //
//  End of Macro for wedge brick arches  //
//                                       //
//---------------------------------------//
